/***************************************************
*	Module Name		:	Clcok_PCF8563_top		   
*	Engineer			:	小梅哥
*	Target Device	:	10CL025YU256C8
*	Tool versions	:	Quartus Prime 18.1
*	Create Date		:	2021-12-20
*	Revision			:	v1.0
*	Description		:  /*使用说明
	1、下载sof程序，数码管默认显示从PCF8563中读取到的时间信息
	2、按下按键S1，切换到显示日期数据上，释放S1切回显示时间
	3、按下按键S0，将预设时间（2019.01.01.01 12:00:00）写入PCF8563
**************************************************/

module Clcok_PCF8563_top(
	Clk,
	Rst_n,
	Key,

	i2c_sclk,
	i2c_sdat,
	
	SH_CP,
	ST_CP,
	DS
); 

	input Clk;
	input Rst_n;
	input [1:0]Key;
	
	output i2c_sclk;
	inout i2c_sdat;
	
	output SH_CP;	//shift clock
	output ST_CP;	//latch data clock
	output DS;	//shift serial data
	
	wire Set_Time;
	wire [23:0]Time_to_Set;
	wire Set_Date;
	wire [31:0]Date_to_Set;
	wire Read;
	wire [23:0]Time_Read;
	wire [31:0]Date_Read;
	
	wire Cmd_Done;
	wire key_flag;
	wire key_state;
	
	//PCF8563驱动
	PCF8563 PCF8563(
		.Clk(Clk),
		.Rst_n(Rst_n),		
		.Set_Time(Set_Time),
		.Time_to_Set(Time_to_Set),
		.Set_Date(Set_Date),
		.Date_to_Set(Date_to_Set),
		.Read(Read),
		.Time_Read(Time_Read),
		.Date_Read(Date_Read),
		.Cmd_Done(Cmd_Done),
		.i2c_sclk(i2c_sclk),
		.i2c_sdat(i2c_sdat)
	);
	
	//独立按键消抖模块
	key_filter key_filter0(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.key_in(Key[0]),
		.key_flag(key_flag),
		.key_state(key_state)
	);
	
	//数码管显示驱动
	wire [31:0]disp_data;
	HEX_top HEX_top(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.disp_data(disp_data),
		.SH_CP(SH_CP),
		.ST_CP(ST_CP),
		.DS(DS)
	);
	
	assign Set_Time = key_flag & (!key_state);	//按下写入预设时间
	assign Set_Date = key_flag & (key_state);		//释放写入预设日期
	
	assign Time_to_Set = 24'h12_00_00;	//时间为12:00:00
	assign Date_to_Set = 32'h19_01_02_01;	//19年01月周二01日
	
	wire [31:0]Time_to_Disp;	//需要送到数码管上显示的带格式的时间数据
	wire [31:0]Date_to_Disp;	//需要送到数码管上显示的带格式的日期数据
	
	assign Time_to_Disp = {Time_Read[23:16], 4'hf, Time_Read[15:8], 4'hf, Time_Read[7:0]};	//f对应显示-
	assign Date_to_Disp = {8'h20,Date_Read[31:24],Date_Read[23:16],Date_Read[7:0]};
	
	//按下按键1可以切换到显示日期
	assign disp_data = Key[1]? Time_to_Disp : Date_to_Disp;

	//设定一个定时器，大概每20ms读取一次PCF8563的日期和时间寄存器	
	reg [19:0]cnt;
	
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		cnt <= 0;
	else
		cnt <= cnt + 1'b1;
		
	assign Read = (cnt == 20'hfffff);
	
endmodule
