/***************************************************
*	Module Name		:	rgmii_udp_loopback_test		   
*	Engineer			:	小梅哥
*	Target Device	:	10CL025YU256C8
*	Tool versions	:	Quartus Prime 18.1
*	Create Date		:	2021-12-20
*	Revision			:	v1.0
*	Description		:  rgmii以太网回环实验顶层模块
**************************************************/

module rgmii_udp_loopback_test(
	//system clock reset
	input clk , //系统时钟输入，50M
	input rst_n , //复位信号输入，低有效
	
	//eth receive interface
	input eth_rxc , //以太网接收时钟
	input [3:0] eth_rxd , //以太网接收数据
	input eth_rxdv , //以太网接收数据有效标志
	
	//eth send interface
	output eth_gtxc , //以太网发送时钟
	output [3:0] eth_txd , //以太网发送数据
	output eth_txen , //以太网发送数据有效标志
	
	// mdio
	output eth_rst_n , //以太网复位，低有效
	inout eth_mdc ,
	inout eth_mdio
);
	wire [7:0]fifo_wrdata;
	wire fifo_wrreq;

	wire clk125M_o;
	wire [15:0]rx_data_length;
	wire one_pkt_done;
	
	//上电后先清零fifo
	reg [23:0]delay_cnt;
	always@(posedge clk or negedge rst_n)
	begin
		if(!rst_n)
			delay_cnt <= 24'd0;
		else if(delay_cnt >= 24'hfffffd)
			delay_cnt <= delay_cnt;
		else
			delay_cnt <= delay_cnt + 1'd1;
	end
	
	wire fifo_aclr;
	assign fifo_aclr =(delay_cnt >= 24'd100)?1'b0:1'b1;
	
	wire phy_init;	//phy initial done
	phy_config phy_config_inst(
		.clk(clk),
		.rst_n(rst_n),
		.phy_rst_n(eth_rst_n),
		.mdc(eth_mdc),
		.mdio(eth_mdio),
//		.rddata(),
		.phy_init(phy_init)
);
	
	rx_pll rx_pll(
		.inclk0(eth_rxc),
		.c0(gmii_rxc)
	);
		
	wire       gmii_rxc;
	wire [7:0] gmii_rxd;
	wire       gmii_rxdv;
	rgmii_to_gmii u_rgmii_to_gmii(
		.rgmii_rxc(gmii_rxc),
		.rgmii_rxd(eth_rxd),
		.rgmii_rxdv(eth_rxdv),
		.gmii_rxc(),
		.gmii_rxd(gmii_rxd),
		.gmii_rxdv(gmii_rxdv)
);
	
	udp_gmii_rx udp_gmii_rx(
		.reset_n(rst_n),
		
		.gmii_rx_en(1'b1),
		
		.local_mac(48'h00_07_ed_ac_62_00),
		.local_ip(32'hc0_a8_00_02),
		.local_port(16'd5000),
		
		.fifo_full(0),
		.fifo_wr(fifo_wrreq),
		.fifo_din(fifo_wrdata),
		.clk125M_o(clk125M_o),
		
		.exter_mac(),
		.exter_ip(),
		.exter_port(),
		.rx_data_length(rx_data_length),
		
		.one_pkt_done(one_pkt_done),
		.pkt_error(),
		.debug_cal_crc(),
		
		.gmii_rx_clk(gmii_rxc),
		.gmii_rxdv(gmii_rxdv),
		.gmii_rxd(gmii_rxd)
);
	
	wire       gmii_gtxc;
	wire [7:0] gmii_txd;
	wire       gmii_txen;
	assign gmii_gtxc = gmii_rxc;
	UDP_Send UDP_Send(
		.Clk(),
		.GMII_GTXC(gmii_gtxc),
		.GMII_TXD(gmii_txd),
		.GMII_TXEN(gmii_txen),
		.Rst_n(rst_n),
		.Go(one_pkt_done),
		.Tx_Done(),
		.data_length(rx_data_length),
		//.des_ip(32'hff_ff_ff_ff),
		.des_ip(32'hc0_a8_00_03),
		.des_mac(48'hFF_FF_FF_FF_FF_FF),
		.des_port(16'd6102),
		.src_ip(32'hc0_a8_00_02),
		.src_mac(48'h00_07_ed_ac_62_00),
		.src_port(16'd5000),
		.wrclk(clk125M_o),
		.wrdata(fifo_wrdata),
		.wrreq(fifo_wrreq),
		.aclr(fifo_aclr),
		.wrusedw()
);
	
	gmii_to_rgmii u_gmii_to_rgmii(
		.gmii_gtxc(gmii_gtxc),
		.gmii_txd(gmii_txd),
		.gmii_txen(gmii_txen),
		.rgmii_gtxc(eth_gtxc),
		.rgmii_txd(eth_txd),
		.rgmii_txen(eth_txen)
);
    
endmodule
