/***************************************************
*	Module Name		:	Key_Board_Test		   
*	Engineer			:	小梅哥
*	Target Device	:	10CL025YU256C8
*	Tool versions	:	Quartus Prime 18.1
*	Create Date		:	2021-12-20
*	Revision			:	v1.0
*	Description		: 矩阵键盘数据hex8数码管移位输出
**************************************************/

module Key_Board_Test(
	Clk,
	Rst_n,
	Key_Board_Row_i,
	Key_Board_Col_o,
	SH_CP,
	ST_CP,
	DS
);

	input Clk;	//50M
	input Rst_n;
	input [3:0]Key_Board_Row_i;
	
	output [3:0]Key_Board_Col_o;
	output SH_CP;	//shift clock
	output ST_CP;	//latch data clock
	output DS;	//shift serial data
	
	wire [7:0] sel;//数码管位选（选择当前要显示的数码管）
	wire [6:0] seg;//数码管段选（当前要显示的内容）	

	wire Key_Flag;
	wire [3:0]Key_Value;
	reg [31:0] disp_data;
	
	Key_Board Key_Board(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.Key_Board_Row_i(Key_Board_Row_i),
		.Key_Board_Col_o(Key_Board_Col_o),
		.Key_Flag(Key_Flag),
		.Key_Value(Key_Value)
	);
	
	
	always@(posedge Clk or negedge Rst_n)
	if(!Rst_n)
		disp_data <= 32'h00000000;
	else if(Key_Flag)
		disp_data <= {disp_data[27:0],Key_Value};

		
	HEX8 HEX8(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.En(1'b1),
		.disp_data(disp_data),
		.sel(sel),
		.seg(seg)
	);
	
	
	HC595_Driver HC595_Driver(
		.Clk(Clk),
		.Rst_n(Rst_n),
		.Data({1'b1,seg,sel}),
		.S_EN(1'b1),
		.SH_CP(SH_CP),
		.ST_CP(ST_CP),
		.DS(DS)
	);
	
endmodule